/*
    $Id: eonlink.c,v 1.7 2003/01/12 14:56:59 cramer Exp $

    Downloads data from a Suunto EON Dive Computer

    Copyright (C) 2001, 2002 Matthias Cramer, cramer@freestone.net
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include <fcntl.h>
#include <sys/termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <strings.h>
#include <string.h>
#include <popt.h>

#include "version.h"
#include "config.h"
#include "eon.h"
#include "progressbar.h"
              
/*
 *  Function prototypes
 */

int openlink(char * device);
void usage (poptContext optCon, int exitcode, char *error, char *addl);

int main(int argc, const char **argv) {
  
  int fd, fd2;
  fd_set rfds;
  struct timeval tv;
  int rec;
  int count;
  unsigned char buf[1];
  unsigned char temp[255];
  char * progress;
  unsigned char setint=0;

  struct s_config config;

  /* Command line parsing arguments */
  char c;
  poptContext optCon;
  char debug=0;
  char verbose=0;
  int interval;
  
  const struct poptOption optionsTable[] = {
                { "time",    't', POPT_ARG_INT, &interval, 'i', "Interval can be 20, 30 or 60 seconts ", "interval"},
                { "interval",'i', POPT_ARG_INT, &interval, 'i', "Interval can be 20, 30 or 60 seconts ", "interval"},
                { "verbose", 'v', POPT_ARG_NONE, 0, 'v', "verbose output", "v" },
                { "debug",   'd', POPT_ARG_NONE, 0, 'd', "debugging output", "d" },
                POPT_AUTOHELP
                { NULL, '\0', 0, NULL, 0 }
  };

  optCon = poptGetContext(NULL, argc, argv, optionsTable, 0);

  readconfig(&config);

  printf("eonlink V%s %s\n", VERSION, COPY);

  /* Now do options processing */
  while ((c = poptGetNextOpt(optCon)) >= 0) {
    switch (c) {
      case 'v': 
        verbose=1;
        break;
      case 'd':
        debug=1;
        break;
      case 'i':
        setint=1;
    }
  }  
     
  if (c < -1) {
     /* an error occurred during option processing */
     fprintf(stderr, "%s: %s\n",
     poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
     poptStrerror(c));
     return 1;
  }

  /*
   if (argc == 2) {
     switch (argv[1][1]) {
       case 'h':
       case '?':
         printf("\nUsage: eonlink [-t interval]\n");
         printf("\n       Interval can be 20, 30 or 60 seconds\n");
         exit(0);
     }
   }
   */

  fd = openlink(config.eondevice);
  
   if (fd < 1) {
     printf("Error opening serial port!!");
     return(-1);
   }

   printf("\nPut your EON into LOG mode before starting this program.\n");
   printf("To abort press Ctrl-C !\n\n");

   fd2=open("eondump.raw",O_WRONLY | O_CREAT | O_TRUNC, 0644);

   if ( setint == 1 ) {
     temp[0]='T';
     switch (interval) {
       case 20:
         temp[1]=20;
         write(fd,temp,2);
         printf("Interval should now be %ds ...\n",interval);
         break;
       case 30:
         temp[1]=30;
         write(fd,temp,2);
         printf("Interval should now be %ds ...\n",interval);
         break;
       case 60:
         temp[1]=60;
         write(fd,temp,2);
         printf("Interval should now be %ds ...\n",interval);
         break;
       default:
         printf("Only intervals of 20, 30, 60 seconds are valid !\n");
     }
   }
   
   temp[0]='P';
   write(fd,temp,1);
   count=0;
   
   printf("Downloading data\n\n");
   progress = progressbar(0,BUFSIZE,0);
   
   FD_ZERO(&rfds);
   FD_SET(fd, &rfds);

   while(count<BUFSIZE) {
     rec=0;
  
     tv.tv_sec=3;
     tv.tv_usec=0;
     
     if (!select(fd+1, &rfds, NULL, NULL, &tv)) {
       printf("\nError: transmission timeout, pleas check connection!\n");
       exit(1);
     }

     rec=read(fd,buf,1);
     if (rec>0) {
       count+=rec;

       if (count % 2 ==0) {
         progress = progressbar(0,BUFSIZE,count);
         printf("\r%s", progress);
       }

       write(fd2, &buf ,1);
     }
   }
   progress = progressbar(0,BUFSIZE,count);
   printf("\n\n");
   close(fd);
   close(fd2);
   return(0);
}

int openlink(char * device) {
  struct termios settings;
  int fd;
  int modelines=0;

   fd=open(device, O_RDWR|O_NOCTTY);
   if(fd==-1) {
     fprintf(stderr,"Device %s cannot be opened!\n",device);
     return(fd);
   }

   if(!isatty(fd)) {
     fprintf(stderr,"Device %s is not a terminal-device !\n",device);
     return(-1);
   }
   
   /*Set inputflags*/
   settings.c_iflag=   IGNBRK  /* Ignore Input breaks */
                     | IGNPAR; /* No Parity  */


   /*Set outputflags*/
   settings.c_oflag=0;         /* nothing special */

   /*Set controlflags*/
   settings.c_cflag= CS8     /* 8 bits per byte */
                   | CREAD   /* enable receiver */
                   | CLOCAL  /* Ignore modem status, line is local */
                   | CSTOPB; /* 2 stoppbits */

   /*Set local flags*/
   settings.c_lflag=0;         /* nothing special */  

   /*Set timeout*/
   settings.c_cc[VTIME]=20;  /* set timeout */

   /*Set minimal bytes to read*/
   settings.c_cc[VMIN]=1;
   
   /*Set baud rate*/
   if(cfsetspeed(&settings,B1200)!=0) {
     fprintf(stderr,"Failed to set baud rate on device %s !\n" ,device);
     return(-1);
   }

   /*Set parameters defined above to the serial device*/
   if(tcsetattr(fd,TCSANOW,&settings)==-1) {
     fprintf(stderr,"Failed to set terminal attributes on device %s !\n",device);
     return(-1);
   }

   /*Read io-controll parameters*/
   if(ioctl(fd,TIOCMGET,&modelines)==-1) {
     fprintf(stderr,"Error doing ioctl TIOCMGET on device %s!\n",device);
     return(-1);
   }

   /*Set RTS*/
   modelines &= ~TIOCM_RTS;
   if(ioctl(fd,TIOCMSET,&modelines)==-1) {
     fprintf(stderr,"Error doing ioctl TIOCMSET on device %s !\n",device); 
     return(-1);
   }
  return(fd);
}

void usage(poptContext optCon, int exitcode, char *error, char *addl) {
  poptPrintUsage(optCon, stderr, 0);   
  if (error) fprintf(stderr, "%s: %s0", error, addl);
  exit(exitcode);
}
